@@ -67,7 +67,7 @@ the file format.
     from Bio import SeqIO
     records = ...
     handle = open("example.faa", "w")
-    alignment = SeqIO.write(records, handle, "fasta")
+    SeqIO.write(records, handle, "fasta")
     handle.close()
 
 In general, you are expected to call this function once (with all your records)
@@ -87,16 +87,14 @@ For sequential files formats (e.g. fasta, genbank) each "record block" holds a
 single sequence.  For these files it would probably be safe to call write()
 multiple times.
 
-If you are using a sequential file format, you may want to write out the records
-one at a time.  To do this, you would ideally create a sequence writer directly...
-
 File Formats
 ============
 When specifying formats, use lowercase strings.
 
 Old Files
 =========
-The modules Bio.SeqIO.FASTA and Bio.SeqIO.generic are considered to be depreciated
+The modules Bio.SeqIO.FASTA and Bio.SeqIO.generic are depreciated and may be
+removed.
 """
 
 #TODO
@@ -204,6 +202,8 @@ def write(sequences, handle, format) :
         raise ValueError("Format required (lower case string)")
     if format <> format.lower() :
         raise ValueError("Format string '%s' should be lower case" % format)
+    if isinstance(sequences,SeqRecord):
+        raise ValueError("Use a SeqRecord list/iterator, not just a single SeqRecord")
 
     #Map the file format to a writer class
     try :
@@ -318,14 +318,15 @@ def to_alignment(sequences, alphabet=generic_alphabet, strict=True) :
             elif alignment_length <> len(record.seq) :
                 raise ValueError("Sequences of different lengths")
             
-            #ToDo, check alphabet for this sequence matches that
-            #specified for the alignment.  Not sure how the
-            #alphabet.contains() method is intended to be used,
-            #but it doesn't make sense to me right now.
+            if not isinstance(record.seq.alphabet, alphabet.__class__) :
+                raise ValueError("Incompatible sequence alphabet")
+            
+            #ToDo, additional checks on the specified alignment...
+            #Should we look at the alphabet.contains() method?
             
         #This is abusing the "private" records list,
         #we should really have a method like add_sequence
-        #but which takes SeqRecord objects.
+        #but which takes SeqRecord objects.  See also Bug 1944
         alignment._records.append(record)
     return alignment
            
