@@ -4,7 +4,6 @@
 import os
 
 import Bio.Geo
-from Bio.File import SGMLHandle
 
 testfiles   = [  'GSE16.txt', 'GSM645.txt', 'GSM691.txt', 'GSM700.txt', 'GSM804.txt'  ]
 
@@ -19,14 +18,9 @@ testfiles.extend(['soft_ex_affy.txt',
 for file in testfiles:
     fh = open(os.path.join("Geo", file))
     print "Testing Bio.Geo on " + file + "\n\n"
-    records = Bio.Geo.Iterator( fh, Bio.Geo.RecordParser(debug_level=0))
-    while 1:
-        record = records.next()
-        if record is not None:
-            print record
-            pass
-        else:
-            break
+    records = Bio.Geo.parse(fh)
+    for record in records:
+        print record
     print "\n"
 
 
